using System;
namespace symmetry
{
	
	/// <summary> Abstract implementation of force functions in a force simulation. This
	/// skeletal version provides support for storing and retrieving float-valued
	/// parameters of the force function. Subclasses should use the protected
	/// field <code>params</code> to store parameter values.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefusex(AT)jheer.org
	/// </author>
	public abstract class AbstractForce : Force
	{
		virtual public int ParameterCount
		{
			get
			{
				return (params_Renamed == null?0:params_Renamed.Length);
			}
			//
			
		}
		protected internal abstract System.String[] ParameterNames{get;}
		virtual public bool ItemForce
		{
			get
			{
				return false;
			}
			//
			
		}
		
		protected internal float[] params_Renamed;
		
		/// <summary> Initialize this force function. This default implementation does nothing.
		/// Subclasses should override this method with any needed initialization.
		/// </summary>
		/// <param name="fsim">the encompassing ForceSimulator
		/// </param>
		public virtual void  init(ForceSimulator fsim)
		{
			// do nothing.
		} //
		
		public virtual float getParameter(int i)
		{
			if (i < 0 || params_Renamed == null || i >= params_Renamed.Length)
			{
				throw new System.IndexOutOfRangeException();
			}
			else
			{
				return params_Renamed[i];
			}
		} //
		
		public virtual System.String getParameterName(int i)
		{
			System.String[] pnames = ParameterNames;
			if (i < 0 || pnames == null || i >= pnames.Length)
			{
				throw new System.IndexOutOfRangeException();
			}
			else
			{
				return pnames[i];
			}
		} //
		
		public virtual void  setParameter(int i, float val)
		{
			if (i < 0 || params_Renamed == null || i >= params_Renamed.Length)
			{
				throw new System.IndexOutOfRangeException();
			}
			else
			{
				params_Renamed[i] = val;
			}
		} //
		
		public virtual bool isSpringForce()
		{
			return false;
		} //
		
		public virtual void  getForce(ForceItem item)
		{
			throw new System.NotSupportedException("This class does not support this operation");
		} //
		
		public virtual void  getForce(Spring spring)
		{
			throw new System.NotSupportedException("This class does not support this operation");
		} //
	} // end of abstract class AbstractForce
}